
#ifndef PAM_CRYPT_H
#define PAM_CRYPT_H

/* MD5 Stuff */

#ifdef __alpha
typedef unsigned int uint32;
#else
typedef unsigned long uint32;
#endif

struct MD5Context {
    uint32 buf[4];
    uint32 bits[2];
    unsigned char in[64];
};

void GoodMD5Init(struct MD5Context *);
void GoodMD5Update(struct MD5Context *, unsigned const char *, unsigned);
void GoodMD5Final(unsigned char digest[16], struct MD5Context *);
void GoodMD5Transform(uint32 buf[4], uint32 const in[16]);
void BrokenMD5Init(struct MD5Context *);
void BrokenMD5Update(struct MD5Context *, unsigned const char *, unsigned);
void BrokenMD5Final(unsigned char digest[16], struct MD5Context *);
void BrokenMD5Transform(uint32 buf[4], uint32 const in[16]);

char *Goodcrypt_md5(const char *pw, const char *salt);
char *Brokencrypt_md5(const char *pw, const char *salt);

/*
* This is needed to make RSAREF happy on some MS-DOS compilers.
*/

typedef struct MD5Context MD5_CTX;

/* SHA1 */

struct SHA1Context {
	uint32 state[5];
	uint32 count[2];
	unsigned char buffer[64];
};

typedef struct SHA1Context SHA1_CTX;

void SHA1Transform(uint32 state[5], unsigned char buffer[64]);
void SHA1Init(SHA1_CTX * context);
void SHA1Update(SHA1_CTX * context, unsigned const char *data, unsigned int len);
void *SHA1Final(unsigned char digest[20], SHA1_CTX * context);

char *crypt_sha1(const char *pw, const char *salt);

/* Bigcrypt */

/*
 * This function implements the "bigcrypt" algorithm specifically for
 * Linux-PAM.
 *  
 * This algorithm is algorithm 0 (default) shipped with the C2 secure
 * implementation of Digital UNIX.
 */

char *bigcrypt(const char *key, const char *salt);

#ifdef PAM_STRONG_CRYPTO
/* Ultrix Crypt16 stuff */

char *crypt16(const char *key, const char *salt);

/* OpenBSD Blowfish stuff */

#define BLOWFISH_ROOT_ROUNDS	8
#define BLOWFISH_USER_ROUNDS	6

char *bcrypt(const char *key, const char *salt);
char *bcrypt_gensalt(u_int8_t log_rounds);

#endif /* PAM_STRONG_CRYPTO */

#endif /* PAM_CRYPT_H */
